"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.process = exports.transform = void 0;
const fs = __importStar(require("fs"));
const globby = __importStar(require("globby"));
const prettier = __importStar(require("prettier"));
const debug_1 = __importDefault(require("debug"));
const debug = debug_1.default('vue-codemod:rule');
const globalAddConfig = {
    global: {
        add: {},
        update: {
            vue: '^3.1.1',
            vuex: '^4.0.1',
            'vue-router': '^4.0.8',
            'vue-i18n': '^9.1.6'
        },
        delete: { 'vue-template-compiler': '', '@vue/composition-api': '' }
    },
    dependencies: {
        add: {},
        update: {},
        delete: {}
    },
    peerDependencies: { add: {}, update: {}, delete: {} },
    devDependencies: {
        add: {
            '@babel/core': '^7.14.6',
            eslint: '^7.20.0',
            '@vue/compiler-sfc': '^3.1.1',
            'eslint-plugin-vue': '^7.11.1'
        },
        update: {
            '@vue/cli-plugin-babel': '^4.5.0',
            '@vue/cli-plugin-eslint': '^4.5.0',
            '@vue/cli-service': '^4.5.0'
        },
        delete: { 'babel-eslint': '' }
    }
};
/**
 * Creates a fix command that inserts text at the specified index in the source text.
 * @param {int} index The 0-based index at which to insert the new text.
 * @param {string} text The text to insert.
 * @returns {Object} The fix command.
 * @private
 */
function transform() {
    debug('Find package.json.');
    const resolvedPaths = globby.sync('package.json');
    if (resolvedPaths.length <= 0) {
        console.warn('package.json is not exists.');
        return false;
    }
    let packageObj = JSON.parse(fs.readFileSync(resolvedPaths[0]).toString());
    packageObj = process(packageObj);
    let formatted = prettier.format(JSON.stringify(packageObj), Object.assign({ parser: 'json' }, (packageObj === null || packageObj === void 0 ? void 0 : packageObj.prettier) ? packageObj === null || packageObj === void 0 ? void 0 : packageObj.prettier : {}));
    fs.writeFileSync(resolvedPaths[0], formatted);
    return true;
}
exports.transform = transform;
/**
 * Modify the configuration of dependencies
 * @param packageObj package.json source
 */
function process(packageObj) {
    var _a, _b;
    Object.keys(globalAddConfig)
        .filter(key => {
        return key != 'global';
    })
        .forEach(key => {
        if (packageObj[key] != undefined) {
            debug(`Process ${key}`);
            packageObj[key] = processCore(packageObj[key], globalAddConfig.global);
            packageObj[key] = processCore(packageObj[key], globalAddConfig[key]);
        }
    });
    if (((_b = (_a = packageObj === null || packageObj === void 0 ? void 0 : packageObj.eslintConfig) === null || _a === void 0 ? void 0 : _a.parserOptions) === null || _b === void 0 ? void 0 : _b.parser) != undefined) {
        packageObj.eslintConfig.parserOptions.parser = '@babel/eslint-parser';
    }
    return packageObj;
}
exports.process = process;
/**
 * process package.json
 * @param packageObj dependencies...
 * @param config key of config
 * @returns package.json
 */
function processCore(packageObj, config) {
    Object.keys(config.add).forEach(key => {
        packageObj[key] = config.add[key];
    });
    Object.keys(config.update).forEach(key => {
        if (packageObj[key] != undefined) {
            packageObj[key] = config.update[key];
        }
    });
    Object.keys(config.delete).forEach(key => {
        delete packageObj[key];
    });
    return packageObj;
}
