"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformAST = void 0;
const OperationUtils = __importStar(require("../src/operationUtils"));
const parser = __importStar(require("vue-eslint-parser"));
const wrapVueTransformation_1 = __importDefault(require("../src/wrapVueTransformation"));
const transformAST = context => {
    let fixOperations = [];
    const toFixNodes = findNodes(context);
    toFixNodes.forEach(node => {
        fixOperations = fixOperations.concat(fix(node));
    });
    return fixOperations;
};
exports.transformAST = transformAST;
exports.default = wrapVueTransformation_1.default(exports.transformAST);
/**
 * search slot attribute nodes
 *
 * @param context
 * @returns slot attribute nodes
 */
function findNodes(context) {
    const { file } = context;
    const source = file.source;
    const options = { sourceType: 'module' };
    const ast = parser.parse(source, options);
    let toFixNodes = [];
    let root = ast.templateBody;
    parser.AST.traverseNodes(root, {
        enterNode(node) {
            if (node.type === 'VAttribute' && node.key.name === 'slot') {
                toFixNodes.push(node);
            }
        },
        leaveNode(node) { }
    });
    return toFixNodes;
}
/**
 * fix logic
 * @param node
 */
function fix(node) {
    let fixOperations = [];
    const element = node.parent.parent;
    // @ts-ignore
    const slotValue = node.value.value;
    if (element != null &&
        element != undefined &&
        element.type == 'VElement' &&
        element.name == 'template') {
        // template element replace slot="xxx" to v-slot:xxx
        fixOperations.push(OperationUtils.replaceText(node, `v-slot:${slotValue}`));
    }
    else {
        // remove v-slot:${slotValue}
        fixOperations.push(OperationUtils.remove(node));
        // add <template v-slot:${slotValue}>
        let elder = null;
        let hasSlotAttr = false;
        let tmp = element;
        // find template parent
        while (elder == null && tmp != null) {
            hasSlotAttr = false;
            tmp = tmp.parent;
            if (tmp == null || tmp.type != 'VElement' || tmp.name != 'template') {
                continue;
            }
            elder = element;
            tmp.startTag.attributes
                .filter((attr) => attr.type === 'VAttribute' &&
                attr.key.type === 'VIdentifier' &&
                attr.key.name === 'slot')
                .forEach((element) => {
                hasSlotAttr = true;
            });
            if (hasSlotAttr) {
                break;
            }
        }
        if (!hasSlotAttr) {
            fixOperations.push(OperationUtils.insertTextBefore(element, `<template v-slot:${slotValue}>`));
            // add </template>
            fixOperations.push(OperationUtils.insertTextAfter(element, `</template>`));
        }
    }
    return fixOperations;
}
