"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformAST = void 0;
const OperationUtils = __importStar(require("../src/operationUtils"));
const parser = __importStar(require("vue-eslint-parser"));
const wrapVueTransformation_1 = __importDefault(require("../src/wrapVueTransformation"));
const debug_1 = __importDefault(require("debug"));
const debug = debug_1.default('vue-codemod:rule');
/**
 * 每一个实际的规则，需要做以下几件事：
 * 1、findNodes(fileInfo, ast): 寻找匹配规则的节点
 * 2、fix(nodes): 完善匹配节点的增删改逻辑
 * 3、applyFix(fileInfo, tempFixes): 执行fixer，对源码进行增删改，并返回转换后的代码
 * @param context
 */
const transformAST = context => {
    let fixOperations = [];
    const toFixNodes = findNodes(context);
    toFixNodes.forEach(node => {
        // fix(node) 返回的为 Operation 数组，因此用 concat 合并多个数组
        fixOperations = fixOperations.concat(fix(node));
    });
    return fixOperations;
};
exports.transformAST = transformAST;
exports.default = wrapVueTransformation_1.default(exports.transformAST);
/**
 * 定位 slot attribute 节点
 *
 * @param context { file: FileInfo, api: API }
 * @param templateBody
 * @returns 所有的 slot attribute 节点
 */
function findNodes(context) {
    const { file } = context;
    const source = file.source;
    const options = { sourceType: 'module' };
    const ast = parser.parse(source, options);
    let toFixNodes = [];
    let root = ast.templateBody; // 强制类型转换
    parser.AST.traverseNodes(root, {
        enterNode(node) {
            if (node.type === 'VAttribute' &&
                node.key.type === 'VIdentifier' &&
                node.key.name === 'key') {
                toFixNodes.push(node);
            }
        },
        leaveNode(node) { }
    });
    return toFixNodes;
}
/**
 * The repair logic for
 * @param node The Target Node
 */
function fix(node) {
    let fixOperations = [];
    const target = node.parent.parent;
    // The current node has no attribute that is v-for
    let hasTargetAttr = false;
    target.startTag.attributes
        .filter((attr) => attr.type === 'VAttribute' &&
        attr.key.type === 'VDirectiveKey' &&
        (attr.key.name.name === 'if' ||
            attr.key.name.name === 'else-if' ||
            attr.key.name.name === 'else'))
        .forEach(() => {
        hasTargetAttr = true;
    });
    if (!hasTargetAttr) {
        debug('No operator, target element is not if/else-if/else.');
        return fixOperations;
    }
    fixOperations.push(OperationUtils.remove(node));
    return fixOperations;
}
